/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCanvasElement.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCanvasRenderingContext2D.h"
#include "gdom/GdomHTMLCanvasElement.h"
#include "gdom/GdomHTMLElement.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomHTMLCanvasElement.h"
#include "gdom/PrivateGdomHTMLElement.h"
#include "gdom/PrivateIntGdomHTMLCanvasElement.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMHTMLCanvasElement::GDOMHTMLCanvasElement(PassRefPtr<WebCore::HTMLCanvasElement> impl)
    : GDOMHTMLElement(impl)
{
}

gchar *
gdom_html_canvas_element_to_data_url (GdomHTMLCanvasElement *thiz, gchar * type)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLCanvasElement * item = (gdom_html_canvas_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (type, 0);
    WebCore::String _g_type = WebCore::String::fromUTF8(type);
    WebCore::ExceptionCode ec = 0;
    gchar * res = _G_GET_UTF8(item->toDataURL(_g_type, ec));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "HTMLCanvasElement.h"
#include <gdom/GdomHTMLCanvasElement.h>
#include <gdom/PrivateGdomHTMLCanvasElement.h>
#include <gdom/PrivateIntGdomHTMLCanvasElement.h>
#include <gdom/GdomHTMLCanvasElementprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomHTMLCanvasElement, gdom_html_canvas_element, GDOM_TYPE_HTML_ELEMENT);

/*
struct _GdomHTMLCanvasElementPrivate {
    class WebCore::HTMLCanvasElement *item;
};

#define GDOM_HTML_CANVAS_ELEMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_HTML_CANVAS_ELEMENT, GdomHTMLCanvasElementPrivate))
*/
G_END_DECLS
WebCore::HTMLCanvasElement *
gdom_html_canvas_element_get_private(GdomHTMLCanvasElement *request)
{
    GDOMHTMLCanvasElement *res;
    res = static_cast<GDOMHTMLCanvasElement*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::HTMLCanvasElement*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_WIDTH,
    PROP_HEIGHT,
};


static void gdom_html_canvas_element_finalize(GObject* object)
{
    /*GdomHTMLCanvasElement* request = GDOM_HTML_CANVAS_ELEMENT(object);*/
    /*GdomHTMLCanvasElementPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_HTML_ELEMENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_html_canvas_element_parent_class)->finalize(object);
}

static void gdom_html_canvas_element__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLCanvasElement *self = GDOM_HTML_CANVAS_ELEMENT(object);
    WebCore::HTMLCanvasElement * gself = (gdom_html_canvas_element_get_private(self));
    switch (prop_id) {
     case PROP_WIDTH:
    {
         gself->setWidth((g_value_get_long (value)) );
         break;
    }
     case PROP_HEIGHT:
    {
         gself->setHeight((g_value_get_long (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_canvas_element__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLCanvasElement *self = GDOM_HTML_CANVAS_ELEMENT(object);
    WebCore::HTMLCanvasElement * gself = (gdom_html_canvas_element_get_private(self));
    switch (prop_id) {
    case PROP_WIDTH:
    {
         g_value_set_long (value, gself->width());
         break;
    }
    case PROP_HEIGHT:
    {
         g_value_set_long (value, gself->height());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_canvas_element_class_init(GdomHTMLCanvasElementClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_html_canvas_element_finalize;
    g_klass->set_property = gdom_html_canvas_element__set_property_fn;
    g_klass->get_property = gdom_html_canvas_element__get_property_fn;

    g_object_class_install_property (g_klass, PROP_WIDTH,
            g_param_spec_long ("width", /* name */
                "html_canvas_element_width", /* short description */
                "read-write  glong HTMLCanvasElement.width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_HEIGHT,
            g_param_spec_long ("height", /* name */
                "html_canvas_element_height", /* short description */
                "read-write  glong HTMLCanvasElement.height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomHTMLCanvasElementPrivate));*/
}

static void gdom_html_canvas_element_init(GdomHTMLCanvasElement* request)
{
    /*GdomHTMLCanvasElementPrivate* priv = GDOM_HTML_CANVAS_ELEMENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

